import numpy as np
import carray as ca
from time import time

NROWS = 1e6
NCOLS = 100

# A column with NROWS
a = np.arange(NROWS)

# Create a ctable with NCOLS columns and NROWS rows
print "Creating the initial ctable.  Please wait..."
t0 = time()
t = ca.ctable([a+i for i in range(NCOLS)])
print "time for creating the ctable:", round(time() - t0, 3)

# get a numpy structured array out of the ctable
print "Creating a numpy structured array out of the ctable"
t0 = time()
nt = t[:]
print "time for getting the structured array:", round(time() - t0, 3)

print "*** Querying numpy structured arrays..."
t0 = time()
npsum1 = sum(r["f0"] for r in nt[(nt["f0"]<10)])
print "time for simple query (numpy):", round(time() - t0, 3)
 
t0 = time()
npsum2 = sum(r["f0"] for r in nt[(nt["f0"]<10) & (np.sin(nt["f99"])**2 + np.log(nt["f50"]) < 4.5)])
print "time for complex query (numpy):", round(time() - t0, 3)

# Now for ctable queries
print "*** Querying carray ctables..."
t0 = time()
casum1 = sum((r.f0 for r in t.where("f0<10")))
print "time for simple query (carray):", round(time() - t0, 3)

t0 = time()
casum2 = sum(r.f0 for r in t.where("(f0<10) & ((sin(f99)**2 + log(f50)) < 4.5)"))
print "time for complex query (carray):", round(time() - t0, 3)

assert casum1 == npsum1
assert casum2 == npsum2
