# Benchmark to compare the times for computing expressions by using
# eval() on carray/numpy arrays.  Numexpr is needed in order to
# execute this.

import math
import numpy as np
import numexpr as ne
import carray as ca
from time import time

N = 1e7       # the number of elements in x

sexprs = [ "((.25*x + .75)*x - 1.5)*x - 2",
           "(((.25*x + .75)*x - 1.5)*x - 2) < 0",
           #"(x+1) < 0",
           ]

# Initial dataset
x = np.linspace(0,100,N)
#x = np.arange(N)

def compute_ref(sexpr):
    t0 = time()
    out = eval(sexpr)
    print "Time for plain numpy --> %.3f" % (time()-t0,)

    t0 = time()
    out = ne.evaluate(sexpr)
    print "Time for numexpr (numpy container) --> %.3f" % (time()-t0,)

def compute_carray(sexpr, clevel, kernel):
    # Uncomment the line below if you want to play with different threads
    #ca.set_nthreads(ca.ncores//2)
    print "*** carray (using compression clevel = %d):" % clevel
    t0 = time()
    cout = ca.eval(sexpr, vm=kernel, cparams=ca.cparams(clevel))
    print "Time for ca.eval (%s) --> %.3f" % (kernel, time()-t0,),
    print ", cratio (out): %.1f" % (cout.nbytes / float(cout.cbytes))
    #print "cout-->", repr(cout)


if __name__=="__main__":

    print "Creating inputs... %d elements from 0 to 100" % N

    # Do the actual evaluation with different setups
    for sexpr in sexprs:
        print "Evaluating '%s' with 10^%d points" % (sexpr, int(math.log10(N)))
        compute_ref(sexpr)
        for kernel in "python", "numexpr":
            compute_carray(sexpr, clevel=0, kernel=kernel)
            compute_carray(sexpr, clevel=9, kernel=kernel)
